@@ -14,8 +14,13 @@ from fsspec.implementations.cache_mapper import (
     HashCacheMapper,
     create_cache_mapper,
 )
-from fsspec.implementations.cached import CachingFileSystem, LocalTempFile
+from fsspec.implementations.cached import (
+    CachingFileSystem,
+    LocalTempFile,
+    WholeFileCacheFileSystem,
+)
 from fsspec.implementations.local import make_path_posix
+from fsspec.implementations.zip import ZipFileSystem
 from fsspec.tests.conftest import win
 
 from .test_ftp import FTPFileSystem
@@ -284,7 +289,10 @@ def test_workflow(ftp_writable, impl):
     with fs.open("/out", "wb") as f:
         f.write(b"changed")
 
-    assert fs.cat("/out") == b"test"  # old value
+    if impl == "filecache":
+        assert (
+            fs.cat("/out") == b"changed"
+        )  # new value, because we overwrote the cached location
 
 
 @pytest.mark.parametrize("impl", ["simplecache", "blockcache"])
@@ -1014,7 +1022,7 @@ def test_multi_cache(protocol):
 def test_multi_cat(protocol, ftp_writable):
     host, port, user, pw = ftp_writable
     fs = FTPFileSystem(host, port, user, pw)
-    for fn in {"/file0", "/file1"}:
+    for fn in ("/file0", "/file1"):
         with fs.open(fn, "wb") as f:
             f.write(b"hello")
 
@@ -1250,3 +1258,49 @@ def test_cache_size(tmpdir, protocol):
     else:
         # Whole cache directory has been deleted
         assert fs.cache_size() == 0
+
+
+def test_spurious_directory_issue1410(tmpdir):
+    import zipfile
+
+    os.chdir(tmpdir)
+    zipfile.ZipFile("dir.zip", mode="w").open("file.txt", "w").write(b"hello")
+    fs = WholeFileCacheFileSystem(fs=ZipFileSystem("dir.zip"))
+
+    assert len(os.listdir()) == 1
+    with fs.open("/file.txt", "rb"):
+        pass
+
+    # There was a bug reported in issue #1410 in which a directory
+    # would be created and the next assertion would fail.
+    assert len(os.listdir()) == 1
+    assert fs._parent("/any/path") == "any"  # correct for ZIP, which has no leading /
+
+
+def test_write_transaction(tmpdir, m, monkeypatch):
+    called = [0]
+    orig = m.put
+
+    def patched_put(*args, **kwargs):
+        called[0] += 1
+        orig(*args, **kwargs)
+
+    monkeypatch.setattr(m, "put", patched_put)
+    tmpdir = str(tmpdir)
+    fs, _ = fsspec.core.url_to_fs("simplecache::memory://", cache_storage=tmpdir)
+    with fs.transaction:
+        fs.pipe("myfile", b"1")
+        fs.pipe("otherfile", b"2")
+        fs.pipe("deep/dir/otherfile", b"3")
+        with fs.open("blarh", "wb") as f:
+            f.write(b"ff")
+        assert not m.find("")
+
+        assert fs.info("otherfile")["size"] == 1
+        assert fs.info("deep")["type"] == "directory"
+        assert fs.isdir("deep")
+        assert fs.ls("deep", detail=False) == ["/deep/dir"]
+
+    assert m.cat("myfile") == b"1"
+    assert m.cat("otherfile") == b"2"
+    assert called[0] == 1  # copy was done in one go
