@@ -129,6 +129,15 @@ def test_list_cache_with_skip_instance_cache(server):
     assert out == [server + "/index/realfile"]
 
 
+def test_glob_return_subfolders(server):
+    h = fsspec.filesystem("http")
+    out = h.glob(server + "/simple/*")
+    assert set(out) == {
+        server + "/simple/dir/",
+        server + "/simple/file",
+    }
+
+
 def test_isdir(server):
     h = fsspec.filesystem("http")
     assert h.isdir(server + "/index/")
@@ -369,6 +378,14 @@ def test_info(server):
     info = fs.info(server + "/index/realfile")
     assert info["ETag"] == "xxx"
 
+    fs = fsspec.filesystem("http", headers={"give_mimetype": "true"})
+    info = fs.info(server + "/index/realfile")
+    assert info["mimetype"] == "text/html"
+
+    fs = fsspec.filesystem("http", headers={"redirect": "true"})
+    info = fs.info(server + "/redirectme")
+    assert info["url"] == server + "/index/realfile"
+
 
 @pytest.mark.parametrize("method", ["POST", "PUT"])
 def test_put_file(server, tmp_path, method, reset_files):
@@ -549,7 +566,7 @@ async def test_async_walk(server):
     # No maxdepth
     res = []
     async for a in fs._walk(server + "/index"):
-        res.append(a)
+        res.append(a)  # noqa: PERF402
     assert res == [(server + "/index", [], ["realfile"])]
 
     # maxdepth=0
